import { world, system } from "@minecraft/server";
import { blocks, crops } from "./blocks";

const distance = 5;

system.runInterval(() => {
    const players = world.getAllPlayers();
    for (const player of players) {
        const lookingEntity = player.getEntitiesFromViewDirection()[0]?.entity;
        const lookingBlock = player.getBlockFromViewDirection()?.block;
        let titleRaw = "";

        if (lookingEntity) {
            if (lookRange(player.getHeadLocation(), lookingEntity.location)) {
                let entityName = lookingEntity.typeId.split(":")[1];
                let identifier = lookingEntity.typeId.split(":")[0];

                let entityItem = lookingEntity.getComponent("item");
                const healthComponent = lookingEntity.getComponent("health");
                const tameProbability =
                    lookingEntity
                        .getComponent("tameable")
                        ?.probability.toFixed(2) || 0;

                if (!entityItem && entityName !== "xp_orb") {
                    if (
                        healthComponent?.defaultValue >= 0 &&
                        entityName !== "armor_stand"
                    ) {
                        const currentHealth = Math.round(
                            healthComponent.currentValue
                        );
                        const maxHealth = healthComponent.defaultValue;
                        titleRaw += `\n${health(currentHealth, maxHealth)}`;
                    }

                    if (player.isSneaking) {
                        if (tameProbability > 0) {
                            titleRaw += `\n§7Tame Chance: ${tameProbability}`;
                        }
                    }

                    const entityDisplayName =
                        entityName !== "player"
                            ? !lookingEntity.nameTag
                                ? `{"translate":"entity.${
                                      identifier === "minecraft"
                                          ? ""
                                          : `${identifier}:`
                                  }${entityName}.name"}`
                                : `{"text": "${lookingEntity.nameTag}"}`
                            : `{"text": "${lookingEntity.name}"}`;
                    const addonName =
                        identifier === "minecraft"
                            ? `{"text": "\n§9§oMinecraft"}`
                            : `{"text": "\n§9§o"},{"translate": "waio.${identifier}:description.name"}`;

                    player.runCommandAsync(
                        `titleraw @s actionbar {"rawtext":[${entityDisplayName},{"text":"${titleRaw}"},${addonName}]}`
                    );
                }
            }
        } else if (lookingBlock) {
            if (lookRange(player.getHeadLocation(), lookingBlock.location)) {
                let blockName = lookingBlock.typeId.split(":")[1];

                let identifier = lookingBlock.typeId.split(":")[0];
                let permutation = lookingBlock.permutation;
                if (identifier === "minecraft") {
                    const color = permutation.getState("color");
                    const chiselType = permutation.getState("chisel_type");
                    const damage = lookingBlock.permutation.getState("damage");
                    const dirtType = permutation.getState("dirt_type");
                    const growth = permutation.getState("growth");
                    const honeyLevel = permutation.getState("honey_level");
                    const newLeafType = permutation.getState("new_leaf_type");
                    const oldLeafType = permutation.getState("old_leaf_type");
                    const openBit = permutation.getState("open_bit");
                    const outputSubtractBit = permutation.getState(
                        "output_subtract_bit"
                    );

                    const prismarineBlockType = permutation.getState(
                        "prismarine_block_type"
                    );
                    const repeaterDelay =
                        permutation.getState("repeater_delay");
                    const sandType = permutation.getState("sand_type");
                    const sandStoneType =
                        permutation.getState("sand_stone_type");
                    const saplingType = permutation.getState("sapling_type");
                    const stoneBrickType =
                        permutation.getState("stone_brick_type");
                    const stoneType = permutation.getState("stone_type");
                    const strippedBit = permutation.getState("stripped_bit");
                    const tallGrassType =
                        permutation.getState("tall_grass_type");
                    const wallBlockType =
                        permutation.getState("wall_block_type");
                    const woodType = permutation.getState("wood_type");

                    if (
                        [
                            "oak_log",
                            "spruce_log",
                            "birch_log",
                            "jungle_log",
                            "acacia_log",
                            "dark_oak_log",
                        ].includes(blockName)
                    ) {
                        blockName = `log.${
                            lookingBlock.getTags()[2] === "dark_oak"
                                ? "big_oak"
                                : lookingBlock.getTags()[2]
                        }`;
                    } else if (
                        ["planks", "wood", "wooden_slab"].includes(blockName)
                    ) {
                        blockName = strippedBit
                            ? blockName + ".stripped"
                            : blockName;
                        blockName +=
                            woodType === "dark_oak" &&
                            blockName !== "wood" &&
                            !strippedBit
                                ? ".big_oak"
                                : `.${woodType}`;
                    } else if (blockName === "cobblestone_wall") {
                        blockName +=
                            wallBlockType === "cobblestone"
                                ? ".normal"
                                : wallBlockType === "mossy_cobblestone"
                                ? ".mossy"
                                : `.${wallBlockType}`;
                    } else if (blockName.includes("fence")) {
                        if (blockName === "oak_fence") {
                            blockName = blockName.replace("oak_fence", "fence");
                        }
                        if (blockName === "dark_oak_fence") {
                            blockName = blockName.replace(
                                "dark_oak_fence",
                                "darkOakFence"
                            );
                        }
                        if (
                            blockName.endsWith("_fence") &&
                            blockName !== "mangrove_fence" &&
                            blockName !== "cherry_fence" &&
                            blockName !== "bamboo_fence" &&
                            blockName !== "nether_brick_fence" &&
                            blockName !== "crimson_fence" &&
                            blockName !== "warped_fence"
                        ) {
                            blockName = blockName.replace("_fence", "Fence");
                        }
                    } else if (
                        [
                            "stained_glass",
                            "stained_glass_pane",
                            "stained_hardened_clay",
                        ].includes(blockName)
                    ) {
                        blockName += `.${
                            color === "light_blue" &&
                            blockName === "stained_hardened_clay"
                                ? "lightBlue"
                                : color
                        }`;
                    } else if (blockName === "stonebrick") {
                        blockName += `.${stoneBrickType}`;
                    } else if (blockName === "prismarine") {
                        blockName += `.${prismarineBlockType}`;
                        if (blockName === "prismarine.default") {
                            blockName = "prismarine.rough";
                        }
                    } else if (
                        blockName === "sandstone" ||
                        blockName === "red_sandstone"
                    ) {
                        blockName += `.${sandStoneType}`;
                        if (
                            blockName === "sandstone.heiroglyphs" ||
                            blockName === "red_sandstone.heiroglyphs"
                        ) {
                            blockName = "sandstone.chiseled";
                        }
                    } else if (
                        blockName === "quartz_block" ||
                        blockName === "purpur_block"
                    ) {
                        blockName += `.${chiselType}`;
                    } else if (blockName.includes("_wool")) {
                        let colorValue = `${
                            blockName === "light_blue_wool" ||
                            blockName === "light_gray_wool"
                                ? blockName === "light_blue_wool"
                                    ? "lightBlue"
                                    : "silver"
                                : blockName.split("_")[0]
                        }`;
                        blockName = `wool.${colorValue}`;
                    } else if (blockName.includes("_carpet")) {
                        let colorValue = `${
                            blockName === "light_blue_carpet" ||
                            blockName === "light_gray_carpet"
                                ? blockName === "light_blue_carpet"
                                    ? "lightBlue"
                                    : "silver"
                                : blockName.split("_")[0]
                        }`;
                        blockName = `carpet.${colorValue}`;
                    } else if (blockName.includes("concrete")) {
                        if (blockName === "concrete_powder") {
                            blockName = `${blockName.replace(
                                "concrete_powder",
                                "concretePowder"
                            )}.${color === "light_blue" ? "lightBlue" : color}`;
                        }

                        if (blockName.includes("_concrete")) {
                            let colorValue = `${
                                blockName === "light_blue_concrete" ||
                                blockName === "light_gray_concrete"
                                    ? blockName === "light_blue_concrete"
                                        ? "lightBlue"
                                        : "silver"
                                    : blockName.split("_")[0]
                            }`;
                            blockName = `concrete.${colorValue}`;
                        }
                    } else if (blockName.includes("glazed_terracotta")) {
                        blockName = blocks[blockName] || blockName;
                    } else if (blockName === "dirt") {
                        blockName += `.${
                            dirtType === "normal" ? "default" : dirtType
                        }`;
                    } else if (blockName == "stone") {
                        blockName += `.${
                            stoneType.includes("_smooth")
                                ? stoneType.replace("_smooth", "Smooth")
                                : stoneType
                        }`;
                    } else if (blockName === "sand") {
                        blockName += `.${
                            sandType === "normal" ? "default" : sandType
                        }`;
                    } else if (
                        blockName === "leaves" ||
                        blockName === "leaves2"
                    ) {
                        blockName += oldLeafType ? `.${oldLeafType}` : "";
                        blockName += newLeafType
                            ? `.${
                                  newLeafType === "dark_oak"
                                      ? "big_oak"
                                      : newLeafType
                              }`
                            : "";
                    } else if (blockName === "sapling") {
                        blockName += `.${
                            saplingType === "dark_oak" ? "big_oak" : saplingType
                        }`;
                    } else if (blockName.includes("double_cut_copper_slab")) {
                        blockName =
                            blockName === "double_cut_copper_slab"
                                ? blockName.replace("double_", "")
                                : blockName.replace("_double", "");
                    } else if (blockName.includes("blackstone_double_slab")) {
                        blockName = blockName.replace("_double", "");
                    } else if (
                        blockName === "polished_blackstone_brick_double_slab"
                    ) {
                        blockName = blockName.replace("_double", "");
                    } else if (blockName.includes("double")) {
                        if (blockName === "double_wooden_slab") {
                            blockName = `${blockName.replace("double_", "")}.${
                                woodType === "dark_oak"
                                    ? "big_oak"
                                    : `${woodType}`
                            }`;
                        } else if (
                            blockName === "mangrove_double_slab" ||
                            blockName === "bamboo_mosaic_double_slab" ||
                            blockName === "bamboo_double_slab"
                        ) {
                            blockName = `${blockName.replace("double_", "")}`;
                        }
                    } else if (blockName === "anvil") {
                        blockName += `${
                            damage === "undamaged"
                                ? ""
                                : `.${damage.replace("_", "")}`
                        }`;
                    } else if (blockName.includes("shulker_box")) {
                        if (blockName === "undyed_shulker_box") {
                            blockName = "shulkerBox";
                        } else {
                            let colorValue = `${
                                blockName === "light_blue_shulker_box" ||
                                blockName === "light_gray_shulker_box"
                                    ? blockName === "light_blue_shulker_box"
                                        ? "lightBlue"
                                        : "silver"
                                    : blockName.split("_")[0]
                            }`;
                            blockName = `shulkerBox${colorValue}`;
                        }
                    } else if (
                        blockName === "bamboo_standing_sign" ||
                        blockName === "cherry_standing_sign" ||
                        blockName === "mangrove_standing_sign"
                    ) {
                        blockName = blockName.replace("_standing", "");
                    } else if (blockName === "tallgrass") {
                        blockName += `${
                            tallGrassType === "fern" ? `.${tallGrassType}` : ""
                        }`;
                    }
                    if (
                        blockName === "lit_deepslate_redstone_ore" ||
                        "lit_redstone_ore"
                    )
                        blockName = blockName.replace("lit_", "");

                    if (blockName === "seagrass") {
                        blockName += ".seagrass";
                    }
                    if (
                        blockName.includes("unpowered_") ||
                        blockName.includes("powered_")
                    ) {
                        blockName = blockName.includes("unpowered")
                            ? blockName.replace("unpowered_", "")
                            : blockName.replace("powered_", "");
                    }

                    if (blockName === "sea_lantern") blockName = "seaLantern";
                    if (blockName === "pitcher_crop")
                        blockName = "pitcher_plant";

                    if (crops.hasOwnProperty(blockName)) {
                        const growthRate = crops[blockName];
                        const result = growth / growthRate;
                        const roundedResult =
                            result !== 0 ? result.toFixed(2) : 0;

                        let growthText =
                            roundedResult === "1.00"
                                ? "§aMature"
                                : `${roundedResult}%`;
                        titleRaw += `\n§7Growth: ${growthText}`;
                    }
                    if (blockName === "melon_stem") {
                        blockName = "melon_block";
                    }

                    if (lookingBlock.getComponent("sign")) {
                        let getText = lookingBlock
                            .getComponent("sign")
                            .getText();

                        if (getText.length > 0) {
                            if (player.isSneaking) {
                                titleRaw += `\n§7Text: ${getText}`;
                            }
                            if (getText.length < 15 && !player.isSneaking) {
                                titleRaw += `\n§7Text: ${getText}`;
                            }
                            if (getText.length >= 15 && !player.isSneaking) {
                                titleRaw += `\n§7Text: ${getText.slice(
                                    0,
                                    14
                                )}...`;
                            }
                        }
                    }

                    if (typeof outputSubtractBit === "boolean") {
                        titleRaw += `\n§7Mode: ${
                            outputSubtractBit ? "Subtractor" : "Comparator"
                        }`;
                    }

                    if (typeof openBit === "boolean" && blockName === "lever") {
                        titleRaw += `\n§7Active: ${openBit}`;
                    }

                    if (typeof repeaterDelay === "number") {
                        titleRaw += `\n§7Delay: ${repeaterDelay + 1}`;
                    }

                    if (lookingBlock.getRedstonePower() > 0) {
                        titleRaw += `\n§7Power: ${lookingBlock.getRedstonePower()}`;
                    }

                    if (typeof honeyLevel === "number") {
                        titleRaw += `\n§7Honey: ${honeyLevel}/5`;
                    }
                }
                if (player.isSneaking) {
                    titleRaw += `\n§7Position: ${lookingBlock.location.x}, ${lookingBlock.location.y}, ${lookingBlock.location.z}`;
                }

                const blockDisplayName = `{"translate":"${
                    !blockName.endsWith("_door") &&
                    blockName !== "brewing_stand" &&
                    blockName !== "bamboo_sign" &&
                    blockName !== "cherry_sign" &&
                    blockName !== "mangrove_sign" &&
                    !blockName.includes("_hanging_") &&
                    blockName !== "frame" &&
                    blockName !== "glow_frame" &&
                    blockName !== "comparator" &&
                    blockName !== "repeater"
                        ? "tile."
                        : "item."
                }${
                    identifier === "minecraft" ? "" : `${identifier}:`
                }${blockName}${
                    blockName === "mud_brick_double_slab" ? "" : `.name`
                }"}`;

                const addonName =
                    identifier === "minecraft"
                        ? `{"text": "\n§9§oMinecraft"}`
                        : `{"text": "\n§9§o"},{"translate": "waio.${identifier}:description.name"}`;

                player.runCommandAsync(
                    `titleraw @s actionbar {"rawtext":[${blockDisplayName},{"text":"${titleRaw}"},${addonName}]}`
                );
            }
        }
    }
}, 1);

function lookRange(headLocation, targetLocation) {
    const { x: headX, y: headY, z: headZ } = headLocation;
    const { x: targetX, y: targetY, z: targetZ } = targetLocation;
    return (
        Math.abs(targetX - headX) < distance &&
        Math.abs(targetY - headY) < distance + 0.2 &&
        Math.abs(targetZ - headZ) < distance
    );
}

function health(currentHealth, maxHealth) {
    const heart = {
        full: "",
        half: "",
        empty: "",
    };

    if (maxHealth <= 20) {
        const totalHearts = Math.ceil(maxHealth / 2);
        const fullHearts = Math.floor(currentHealth / 2);
        const halfHeart = currentHealth % 2 === 1 && fullHearts < totalHearts;
        const emptyHearts = totalHearts - fullHearts - (halfHeart ? 1 : 0);

        return `${heart.full.repeat(fullHearts)}${
            halfHeart ? heart.half : ""
        }${heart.empty.repeat(emptyHearts)}`;
    } else {
        return `§7Health: ${currentHealth}/${maxHealth}`;
    }
}
